@@ -1,15 +1,30 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* e-cell-tree.c - Tree cell renderer
- * Copyright (C) 2000 Ximian, Inc.
+/* 
+ * e-cell-tree.c - Tree cell object.
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Author: Chris Toshok <toshok@ximian.com>
+ * Authors:
+ *   Chris Toshok <toshok@ximian.com>
  *
  * A majority of code taken from:
  *
  * the ECellText renderer.
+ * Copyright 1998, The Free Software Foundation
+ * Copyright 1999, 2000, Ximian, Inc.
  *
- * Copyright (C) 1998 The Free Software Foundation
- * Copyright (C) 1999, 2000 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
@@ -478,6 +493,17 @@ ect_show_tooltip (ECellView *ecell_view, int model_col, int view_col, int row,
 	tooltip->x += offset;
 	e_cell_show_tooltip (tree_view->subcell_view, model_col, view_col, row, col_width - offset, tooltip);
 }
+
+/*
+ * ECellView::get_bg_color method
+ */
+static char *
+ect_get_bg_color (ECellView *ecell_view, int row)
+{		
+	ECellTreeView *tree_view = (ECellTreeView *) ecell_view;
+
+	return e_cell_get_bg_color (tree_view->subcell_view, row);
+}
 		
 /*
  * ECellView::enter_edit method
@@ -635,19 +661,20 @@ e_cell_tree_class_init (GtkObjectClass *object_class)
 
 	object_class->destroy = ect_destroy;
 
-	ecc->new_view   = ect_new_view;
-	ecc->kill_view  = ect_kill_view;
-	ecc->realize    = ect_realize;
-	ecc->unrealize  = ect_unrealize;
-	ecc->draw       = ect_draw;
-	ecc->event      = ect_event;
-	ecc->height     = ect_height;
-	ecc->enter_edit = ect_enter_edit;
-	ecc->leave_edit = ect_leave_edit;
-	ecc->print      = ect_print;
-	ecc->print_height = ect_print_height;
-	ecc->max_width = ect_max_width;
-	ecc->show_tooltip = ect_show_tooltip;
+	ecc->new_view         = ect_new_view;
+	ecc->kill_view        = ect_kill_view;
+	ecc->realize          = ect_realize;
+	ecc->unrealize        = ect_unrealize;
+	ecc->draw             = ect_draw;
+	ecc->event            = ect_event;
+	ecc->height           = ect_height;
+	ecc->enter_edit       = ect_enter_edit;
+	ecc->leave_edit       = ect_leave_edit;
+	ecc->print            = ect_print;
+	ecc->print_height     = ect_print_height;
+	ecc->max_width        = ect_max_width;
+	ecc->show_tooltip     = ect_show_tooltip;
+	ecc->get_bg_color     = ect_get_bg_color;
 
 	parent_class = gtk_type_class (PARENT_TYPE);
 }
