@@ -1,12 +1,27 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table-column-view.c: A canvas item based view of the ETableColumn.
+/* 
+ * e-table-header-item.c
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Author:
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *   Miguel de Icaza (miguel@gnu.org)
  *
- * Copyright 1999, 2000, 2001 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 
 #include "e-table-header-item.h"
@@ -56,6 +71,7 @@ static guint ethi_signals [LAST_SIGNAL] = { 0, };
 #define PARENT_OBJECT_TYPE gnome_canvas_item_get_type ()
 
 #define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
+#define d(x)
 
 static GnomeCanvasItemClass *ethi_parent_class;
 
@@ -195,6 +211,7 @@ ethi_font_set (ETableHeaderItem *ethi, GdkFont *font)
 		gdk_font_unref (ethi->font);
 
 	ethi->font = font;
+	gdk_font_ref (font);
 	
 	ethi->height = e_table_header_item_get_height (ethi);
 	e_canvas_item_request_reflow(GNOME_CANVAS_ITEM(ethi));
@@ -203,13 +220,18 @@ ethi_font_set (ETableHeaderItem *ethi, GdkFont *font)
 static void
 ethi_font_load (ETableHeaderItem *ethi, char *fontname)
 {
-	GdkFont *font;
+	GdkFont *font = NULL;
 
-	font = gdk_fontset_load (fontname);
-	if (font == NULL)
-		font = gdk_font_load ("-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-iso8859-1");
+	if (fontname != NULL)
+		font = gdk_fontset_load (fontname);
+
+	if (font == NULL) {
+		font = GTK_WIDGET (GNOME_CANVAS_ITEM (ethi)->canvas)->style->font;
+		gdk_font_ref (font);
+	}
 	
 	ethi_font_set (ethi, font);
+	gdk_font_unref (font);
 }
 
 static void
@@ -363,21 +385,28 @@ ethi_find_col_by_x (ETableHeaderItem *ethi, int x)
 	const int cols = e_table_header_count (ethi->eth);
 	int x1 = 0;
 	int col;
-
-	if (x < x1)
-		return -1;
+	
+	d(g_print ("%s:%d: x = %d, x1 = %d\n", __FUNCTION__, __LINE__, x, x1));
 
 	x1 += ethi->group_indent_width;
 	
+	if (x < x1) {
+		d(g_print ("%s:%d: Returning 0\n", __FUNCTION__, __LINE__));
+		return 0;
+	}
+
 	for (col = 0; col < cols; col++){
 		ETableCol *ecol = e_table_header_get_column (ethi->eth, col);
 
-		if ((x >= x1) && (x <= x1 + ecol->width))
+		if ((x >= x1) && (x <= x1 + ecol->width)) {
+			d(g_print ("%s:%d: Returning %d\n", __FUNCTION__, __LINE__, col));
 			return col;
+		}
 
 		x1 += ecol->width;
 	}
-	return -1;
+	d(g_print ("%s:%d: Returning %d\n", __FUNCTION__, __LINE__, cols - 1));
+	return cols - 1;
 }
 
 static int
@@ -387,11 +416,11 @@ ethi_find_col_by_x_nearest (ETableHeaderItem *ethi, int x)
 	int x1 = 0;
 	int col;
 
-	if (x < x1)
-		return -1;
-
 	x1 += ethi->group_indent_width;
 	
+	if (x < x1)
+		return 0;
+
 	for (col = 0; col < cols; col++){
 		ETableCol *ecol = e_table_header_get_column (ethi->eth, col);
 
@@ -715,7 +744,6 @@ ethi_realize (GnomeCanvasItem *item)
 {
 	ETableHeaderItem *ethi = E_TABLE_HEADER_ITEM (item);
 	GdkWindow *window;
-	GdkColor c;
 	GtkTargetEntry  ethi_drop_types [] = {
 		{ TARGET_ETABLE_COL_TYPE, 0, TARGET_ETABLE_COL_HEADER },
 	};
@@ -726,10 +754,6 @@ ethi_realize (GnomeCanvasItem *item)
 
 	window = GTK_WIDGET (item->canvas)->window;
 
-	ethi->gc = gdk_gc_new (window);
-	gnome_canvas_get_color (item->canvas, "black", &c);
-	gdk_gc_set_foreground (ethi->gc, &c);
-
 	if (!ethi->font)
 		ethi_font_set (ethi, GTK_WIDGET (item->canvas)->style->font);
 
@@ -765,8 +789,7 @@ ethi_unrealize (GnomeCanvasItem *item)
 {
 	ETableHeaderItem *ethi = E_TABLE_HEADER_ITEM (item);
 
-	gdk_gc_unref (ethi->gc);
-	ethi->gc = NULL;
+	gdk_font_unref (ethi->font);
 
 	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_motion_id);
 	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_leave_id);
@@ -841,7 +864,7 @@ ethi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width
 		e_table_header_draw_button (drawable, ecol,
 					    GTK_WIDGET (canvas)->style, ethi->font,
 					    GTK_WIDGET_STATE (canvas),
-					    GTK_WIDGET (canvas), ethi->gc,
+					    GTK_WIDGET (canvas),
 					    x1 - x, -y,
 					    width, height,
 					    x2 - x1, ethi->height,
@@ -1033,7 +1056,7 @@ ethi_start_drag (ETableHeaderItem *ethi, GdkEvent *event)
 		pixmap, ecol,
 		widget->style, ethi->font,
 		GTK_WIDGET_STATE (widget),
-		widget, ethi->gc,
+		widget,
 		0, 0,
 		col_width, ethi->height,
 		col_width, ethi->height,
@@ -1276,7 +1299,7 @@ ethi_popup_customize_view(GtkWidget *widget, EthiHeaderInfo *info)
 			return;
 
 		ethi->config = e_table_config_new (
-				_("Configuring view: FIXME"),
+				_("Customize Current View"),
 				spec, state);
 		gtk_signal_connect (
 			GTK_OBJECT (ethi->config), "destroy",
