@@ -1,18 +1,34 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table-col-head.c: TableColHead implementation
+/* 
+ * e-table-header.c
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@gnu.org)
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *   Miguel de Icaza <miguel@ximian.com>
  *
- * (C) 1999 Ximian, Inc
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <string.h>
 #include <gtk/gtkobject.h>
 #include <gtk/gtksignal.h>
 #include "e-table-header.h"
 #include "e-table-defines.h"
+#include "gal/util/e-util.h"
 
 /* The arguments we take */
 enum {
@@ -236,26 +252,26 @@ e_table_header_class_init (GtkObjectClass *object_class)
 	eth_signals [STRUCTURE_CHANGE] =
 		gtk_signal_new ("structure_change",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableHeaderClass, structure_change),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 	eth_signals [DIMENSION_CHANGE] = 
 		gtk_signal_new ("dimension_change", 
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableHeaderClass, dimension_change),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
 	eth_signals [REQUEST_WIDTH] = 
 		gtk_signal_new ("request_width",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableHeaderClass, request_width),
 				gtk_marshal_INT__INT,
 				GTK_TYPE_INT, 1, GTK_TYPE_INT);
 
-	gtk_object_class_add_signals (object_class, eth_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, eth_signals, LAST_SIGNAL);
 }
 
 static void
@@ -833,3 +849,26 @@ e_table_header_get_type (void)
 
 	return type;
 }
+
+int
+e_table_header_prioritized_column (ETableHeader *eth)
+{
+	int best_model_col = 0;
+	int best_priority;
+	int i;
+	int count;
+
+	count = e_table_header_count (eth);
+	if (count == 0)
+		return -1;
+	best_priority = e_table_header_get_column (eth, 0)->priority;
+	best_model_col = e_table_header_get_column (eth, 0)->col_idx;
+	for (i = 1; i < count; i++) {
+		int priority = e_table_header_get_column (eth, i)->priority;
+		if (priority > best_priority) {
+			best_priority = priority;
+			best_model_col = e_table_header_get_column (eth, i)->col_idx;
+		}
+	}
+	return best_model_col;
+}
