@@ -288,6 +288,16 @@ et_destroy (GtkObject *object)
 	(*parent_class->destroy)(object);
 }
 
+static void
+et_unrealize (GtkWidget *widget)
+{
+	scroll_off (E_TREE (widget));
+	hover_off (E_TREE (widget));
+
+	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
+		GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+}
+
 static void
 e_tree_init (GtkObject *object)
 {
@@ -595,9 +605,14 @@ item_key_press (ETableItem *eti, int row, int col, GdkEvent *event, ETree *et)
 		y = CLAMP(vadj->value + (2 * vadj->page_size - 50), 0, vadj->upper);
 		y -= vadj->value;
 		e_tree_get_cell_at (et, 30, y, &row_local, &col_local);
+
+		if (row_local == -1)
+			row_local = e_table_model_row_count (E_TABLE_MODEL(et->priv->etta)) - 1;
+
 		row_local = e_tree_view_to_model_row (et, row_local);
 		col_local = e_selection_model_cursor_col (E_SELECTION_MODEL (et->priv->selection));
 		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->priv->selection), row_local, col_local, key->state);
+
 		return_val = 1;
 		break;
 	case GDK_Page_Up:
@@ -606,9 +621,14 @@ item_key_press (ETableItem *eti, int row, int col, GdkEvent *event, ETree *et)
 		y = CLAMP(vadj->value - (vadj->page_size - 50), 0, vadj->upper);
 		y -= vadj->value;
 		e_tree_get_cell_at (et, 30, y, &row_local, &col_local);
+
+		if (row_local == -1)
+			row_local = e_table_model_row_count (E_TABLE_MODEL(et->priv->etta)) - 1;
+
 		row_local = e_tree_view_to_model_row (et, row_local);
 		col_local = e_selection_model_cursor_col (E_SELECTION_MODEL (et->priv->selection));
 		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->priv->selection), row_local, col_local, key->state);
+
 		return_val = 1;
 		break;
 	case '=':
@@ -659,7 +679,14 @@ item_start_drag (ETableItem *eti, int row, int col, GdkEvent *event, ETree *et)
 }
 
 static void
-et_selection_model_selection_change (ETableSelectionModel *etsm, ETable *et)
+et_selection_model_selection_changed (ETableSelectionModel *etsm, ETree *et)
+{
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [SELECTION_CHANGE]);
+}
+
+static void
+et_selection_model_selection_row_changed (ETableSelectionModel *etsm, int row, ETree *et)
 {
 	gtk_signal_emit (GTK_OBJECT (et),
 			 et_signals [SELECTION_CHANGE]);
@@ -860,11 +887,11 @@ e_tree_set_state_object(ETree *e_tree, ETableState *state)
 
 /**
  * e_tree_set_state:
- * @e_tree: %ETree object that will be modified
- * @state_str: a string with the XML representation of the ETableState.
+ * @e_tree: #ETree object that will be modified
+ * @state_str: a string with the XML representation of the #ETableState.
  *
- * This routine sets the state (as described by %ETableState) of the
- * %ETree object.
+ * This routine sets the state (as described by #ETableState) of the
+ * #ETree object.
  */
 void
 e_tree_set_state (ETree      *e_tree,
@@ -887,10 +914,10 @@ e_tree_set_state (ETree      *e_tree,
 
 /**
  * e_tree_load_state:
- * @e_tree: %ETree object that will be modified
- * @filename: name of the file containing the state to be loaded into the %ETree
+ * @e_tree: #ETree object that will be modified
+ * @filename: name of the file containing the state to be loaded into the #ETree
  *
- * An %ETableState will be loaded form the file pointed by @filename into the
+ * An #ETableState will be loaded form the file pointed by @filename into the
  * @e_tree object.
  */
 void
@@ -914,11 +941,14 @@ e_tree_load_state (ETree      *e_tree,
 
 /**
  * e_tree_get_state_object:
- * @e_tree: %ETree object that will be modified
+ * @e_tree: #ETree object to act on
  *
- * Returns: the %ETreeState object that encapsulates the current
- * state of the @e_tree object
- */
+ * Builds an #ETableState corresponding to the current state of the
+ * #ETree.
+ *
+ * Return value:
+ * The %ETableState object generated.
+ **/
 ETableState *
 e_tree_get_state_object (ETree *e_tree)
 {
@@ -950,7 +980,18 @@ e_tree_get_state_object (ETree *e_tree)
 	return state;
 }
 
-gchar          *e_tree_get_state                 (ETree               *e_tree)
+/**
+ * e_tree_get_state:
+ * @e_tree: The #ETree to act on
+ * 
+ * Builds a state object based on the current state and returns the
+ * string corresponding to that state.
+ * 
+ * Return value: 
+ * A string describing the current state of the #ETree.
+ **/
+gchar *
+e_tree_get_state (ETree *e_tree)
 {
 	ETableState *state;
 	gchar *string;
@@ -963,12 +1004,12 @@ gchar          *e_tree_get_state                 (ETree               *e_tree)
 
 /**
  * e_tree_save_state:
- * @e_tree: %ETree object that will be modified
- * @filename: name of the file containing the state to be loaded into the %ETree
+ * @e_tree: The #ETree to act on
+ * @filename: name of the file to save to
  *
- * This routine saves the state of the @e_tree object into the file pointed
- * by @filename
- */
+ * Saves the state of the @e_tree object into the file pointed by
+ * @filename.
+ **/
 void
 e_tree_save_state (ETree      *e_tree,
 		   const gchar *filename)
@@ -980,6 +1021,14 @@ e_tree_save_state (ETree      *e_tree,
 	gtk_object_unref(GTK_OBJECT(state));
 }
 
+/**
+ * e_tree_get_spec:
+ * @e_tree: The #ETree to query
+ * 
+ * Returns the specification object.
+ * 
+ * Return value:
+ **/
 ETableSpecification *
 e_tree_get_spec (ETree *e_tree)
 {
@@ -1095,7 +1144,9 @@ et_real_construct (ETree *e_tree, ETreeModel *etm, ETableExtras *ete,
 			NULL);
 
 	gtk_signal_connect(GTK_OBJECT(e_tree->priv->selection), "selection_changed",
-			   GTK_SIGNAL_FUNC(et_selection_model_selection_change), e_tree);
+			   GTK_SIGNAL_FUNC(et_selection_model_selection_changed), e_tree);
+	gtk_signal_connect(GTK_OBJECT(e_tree->priv->selection), "selection_row_changed",
+			   GTK_SIGNAL_FUNC(et_selection_model_selection_row_changed), e_tree);
 
 	if (!specification->no_headers) {
 		e_tree_setup_header (e_tree);
@@ -1131,6 +1182,20 @@ et_real_construct (ETree *e_tree, ETreeModel *etm, ETableExtras *ete,
 	return e_tree;
 }
 
+/**
+ * e_tree_construct:
+ * @e_tree: The newly created #ETree object.
+ * @etm: The model for this table.
+ * @ete: An optional #ETableExtras.  (%NULL is valid.)
+ * @spec_str: The spec.
+ * @state_str: An optional state.  (%NULL is valid.)
+ * 
+ * This is the internal implementation of e_tree_new() for use by
+ * subclasses or language bindings.  See e_tree_new() for details.
+ * 
+ * Return value: 
+ * The passed in value @e_tree or %NULL if there's an error.
+ **/
 ETree *
 e_tree_construct (ETree *e_tree, ETreeModel *etm, ETableExtras *ete,
 		   const char *spec_str, const char *state_str)
@@ -1168,6 +1233,21 @@ e_tree_construct (ETree *e_tree, ETreeModel *etm, ETableExtras *ete,
 	return e_tree;
 }
 
+/**
+ * e_tree_construct_from_spec_file:
+ * @e_tree: The newly created #ETree object.
+ * @etm: The model for this tree
+ * @ete: An optional #ETableExtras  (%NULL is valid.)
+ * @spec_fn: The filename of the spec
+ * @state_fn: An optional state file  (%NULL is valid.)
+ *
+ * This is the internal implementation of e_tree_new_from_spec_file()
+ * for use by subclasses or language bindings.  See
+ * e_tree_new_from_spec_file() for details.
+ * 
+ * Return value: 
+ * The passed in value @e_tree or %NULL if there's an error.
+ **/
 ETree *
 e_tree_construct_from_spec_file (ETree *e_tree, ETreeModel *etm, ETableExtras *ete,
 				  const char *spec_fn, const char *state_fn)
@@ -1213,6 +1293,27 @@ e_tree_construct_from_spec_file (ETree *e_tree, ETreeModel *etm, ETableExtras *e
 	return e_tree;
 }
 
+/**
+ * e_tree_new:
+ * @etm: The model for this tree
+ * @ete: An optional #ETableExtras  (%NULL is valid.)
+ * @spec: The spec
+ * @state: An optional state  (%NULL is valid.)
+ * 
+ * This function creates an #ETree from the given parameters.  The
+ * #ETreeModel is a tree model to be represented.  The #ETableExtras
+ * is an optional set of pixbufs, cells, and sorting functions to be
+ * used when interpreting the spec.  If you pass in %NULL it uses the
+ * default #ETableExtras.  (See e_table_extras_new()).
+ *
+ * @spec is the specification of the set of viewable columns and the
+ * default sorting state and such.  @state is an optional string
+ * specifying the current sorting state and such.  If @state is NULL,
+ * then the default state from the spec will be used.
+ * 
+ * Return value: 
+ * The newly created #ETree or %NULL if there's an error.
+ **/
 GtkWidget *
 e_tree_new (ETreeModel *etm, ETableExtras *ete, const char *spec, const char *state)
 {
@@ -1234,6 +1335,26 @@ e_tree_new (ETreeModel *etm, ETableExtras *ete, const char *spec, const char *st
 	return (GtkWidget *) ret_val;
 }
 
+/**
+ * e_tree_new_from_spec_file:
+ * @etm: The model for this tree.
+ * @ete: An optional #ETableExtras.  (%NULL is valid.)
+ * @spec_fn: The filename of the spec.
+ * @state_fn: An optional state file.  (%NULL is valid.)
+ * 
+ * This is very similar to e_tree_new(), except instead of passing in
+ * strings you pass in the file names of the spec and state to load.
+ *
+ * @spec_fn is the filename of the spec to load.  If this file doesn't
+ * exist, e_tree_new_from_spec_file will return %NULL.
+ *
+ * @state_fn is the filename of the initial state to load.  If this is
+ * %NULL or if the specified file doesn't exist, the default state
+ * from the spec file is used.
+ * 
+ * Return value: 
+ * The newly created #ETree or %NULL if there's an error.
+ **/
 GtkWidget *
 e_tree_new_from_spec_file (ETreeModel *etm, ETableExtras *ete, const char *spec_fn, const char *state_fn)
 {
@@ -2273,12 +2394,14 @@ static void
 context_destroyed (gpointer data)
 {
 	ETree *et = data;
-	et->priv->last_drop_x       = 0;
-	et->priv->last_drop_y       = 0;
-	et->priv->last_drop_time    = 0;
-	et->priv->last_drop_context = NULL;
-	scroll_off (et);
-	hover_off (et);
+	if (et->priv) {
+		et->priv->last_drop_x       = 0;
+		et->priv->last_drop_y       = 0;
+		et->priv->last_drop_time    = 0;
+		et->priv->last_drop_context = NULL;
+		scroll_off (et);
+		hover_off (et);
+	}
 	gtk_object_unref (GTK_OBJECT (et));
 }
 
@@ -2470,6 +2593,7 @@ e_tree_class_init (ETreeClass *class)
 	object_class->get_arg          = et_get_arg;
 
 	widget_class->grab_focus       = et_grab_focus;
+	widget_class->unrealize        = et_unrealize;
 
 	container_class->focus         = et_focus;
 
