@@ -1,19 +1,33 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * gal-view-collection.c: a View Collection
+/* 
+ * gal-view-collection.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
  * Authors:
- *   Chris Lahey (clahey@helixcode.com)
+ *   Chris Lahey <clahey@ximian.com>
  *
- * (C) 1999, 2000 Helix Code, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 
 #include <util/e-i18n.h>
 #include <ctype.h>
 #include <string.h>
 #include <gtk/gtksignal.h>
-#include <gnome-xml/parser.h>
+#include <parser.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-util.h>
 #include <gal/util/e-util.h>
@@ -109,7 +123,7 @@ gal_view_collection_class_init (GtkObjectClass *object_class)
 	gal_view_collection_signals [DISPLAY_VIEW] =
 		gtk_signal_new ("display_view",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (GalViewCollectionClass, display_view),
 				gtk_marshal_NONE__OBJECT,
 				GTK_TYPE_NONE, 1, GAL_VIEW_TYPE);
@@ -117,12 +131,12 @@ gal_view_collection_class_init (GtkObjectClass *object_class)
 	gal_view_collection_signals [CHANGED] =
 		gtk_signal_new ("changed",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (GalViewCollectionClass, changed),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 
-	gtk_object_class_add_signals (object_class, gal_view_collection_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, gal_view_collection_signals, LAST_SIGNAL);
 
 	klass->display_view = NULL;
 	klass->changed      = NULL;
@@ -264,7 +278,7 @@ load_single_file (GalViewCollection *collection,
 
 		factory = NULL;
 		for (factories = collection->factory_list; factories; factories = factories->next) {
-			if (!strcmp(gal_view_factory_get_type_code(factories->data), item->type)) {
+			if (item->type && !strcmp(gal_view_factory_get_type_code(factories->data), item->type)) {
 				factory = factories->data;
 				break;
 			}
