@@ -1,9 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * Author:
- *   Christopher James Lahey <clahey@helixcode.com>
+/* 
+ * e-categories-master-list-dialog-model.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * (C) 2001 Ximian, Inc.
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
@@ -47,8 +62,11 @@ ecmldm_destroy(GtkObject *object)
 					       model->priv->ecml_changed_id);
 		gtk_object_unref (GTK_OBJECT (model->priv->ecml));
 	}
-	g_free (model->priv->ecml);
-	model->priv->ecml = NULL;
+	g_free (model->priv);
+	model->priv = NULL;
+
+	if (GTK_OBJECT_CLASS (parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
 /* This function returns the number of columns in our ETableModel. */
