@@ -1,17 +1,28 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-#define _E_FONT_C_
-
-/*
- * e-font
+/* 
+ * e-font.c - Temporary wrappers around GdkFonts to get unicode displaying
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Temporary wrappers around GdkFonts to get unicode displaying
+ * Authors:
+ *   Lauris Kaplinski <lauris@ximian.com>
  *
- * Author: Lauris Kaplinski <lauris@helixcode.com>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * Copyright (C) 2000 Helix Code, Inc.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
+#define _E_FONT_C_
+
 #include <config.h>
 #include <stdlib.h>
 #include <string.h>
@@ -21,6 +32,7 @@
 #include "gal/util/e-cache.h"
 #include "e-font.h"
 #include "e-unicode.h"
+#include "gal/util/e-iconv.h"
 
 #ifdef HAVE_CODESET
 #include <langinfo.h>
@@ -63,11 +75,7 @@ static void split_name (gchar * c[], gchar * name);
 static gboolean find_variants (gchar **namelist, gint length,
 			       gchar *base_weight, gchar **light,
 			       gchar **bold);
-static const gchar *get_locale_charset (void);
 static gint         e_font_to_native   (EFont *font, gchar *native, const gchar *utf, gint bytes);
-#if 0
-static const gchar *locale_charset_to_encoding(void);
-#endif
 #ifdef E_FONT_VERBOSE
 static void e_font_print_gdk_font_name (const GdkFont * font);
 #endif
@@ -626,6 +634,8 @@ e_font_unref (EFont *font)
 	font->refcount--;
 
 	if (font->refcount < 1) {
+		e_iconv_close(font->to);
+		e_iconv_close(font->from);
 		gdk_font_unref (font->font);
 		if (font->bold) gdk_font_unref (font->bold);
 		g_free (font);
@@ -713,7 +723,7 @@ replace_nbsp_with_spaces (const guchar *orig, gint *bytes)
 static gint
 e_font_to_native (EFont *font, gchar *native, const gchar *utf, gint bytes)
 {
-	char *ib;
+	const char *ib;
 	char *ob;
 	size_t ibl, obl;
 	gint rv;
@@ -724,24 +734,16 @@ e_font_to_native (EFont *font, gchar *native, const gchar *utf, gint bytes)
 	if (font->to == (iconv_t) -1)
 		rv = no_conv_wrapper (font, native, utf, bytes);
 	else {
-		ib = (char *) utf;
+		ib = utf;
 		ibl = bytes;
 		ob = native;
 		obl = bytes * 4;
 
 		while (ibl > 0) {
-			iconv (font->to, &ib, &ibl, &ob, &obl);
+			e_iconv (font->to, &ib, &ibl, &ob, &obl);
 			if (ibl > 0) {
-				gint len;
-				if ((*ib & 0x80) == 0x00) len = 1;
-				else if ((*ib &0xe0) == 0xc0) len = 2;
-				else if ((*ib &0xf0) == 0xe0) len = 3;
-				else if ((*ib &0xf80) == 0xf0) len = 4;
-				else {
-					g_warning ("Invalid UTF-8 sequence");
-					break;
-				}
-				ib += len;
+				ib = g_utf8_next_char (ib);
+
 				ibl = bytes - (ib - utf);
 				if (ibl > bytes) ibl = 0;
 				if (!font->twobyte) {
@@ -817,176 +819,10 @@ e_font_utf8_text_width (EFont *font, EFontStyle style, const char *text, int num
 gint
 e_font_utf8_char_width (EFont *font, EFontStyle style, char *text)
 {
-	gint len;
-
 	g_return_val_if_fail (font != NULL, 0);
 	g_return_val_if_fail (text != NULL, 0);
 
-	if ((*text & 0x80) == 0x00) len = 1;
-	else if ((*text &0xe0) == 0xc0) len = 2;
-	else if ((*text &0xf0) == 0xe0) len = 3;
-	else if ((*text &0xf80) == 0xf0) len = 4;
-	else {
-		g_warning ("Invalid UTF-8 sequence");
-		return 0;
-	}
-
-	return e_font_utf8_text_width (font, style, text, len);
-}
-
-static const gchar *
-translate_encoding (const gchar *encoding)
-{
-	static GHashTable *eh = NULL;
-	gchar e[64];
-	gchar *finalenc;
-
-	if (eh == NULL) {
-		eh = g_hash_table_new (g_str_hash, g_str_equal);
-
-		g_hash_table_insert (eh, "iso8859-1", "iso-8859-1");
-		g_hash_table_insert (eh, "iso_8859-1", "iso-8859-1");
-		g_hash_table_insert (eh, "iso8859-2", "iso-8859-2");
-		g_hash_table_insert (eh, "iso_8859-2", "iso-8859-2");
-		g_hash_table_insert (eh, "iso8859-3", "iso-8859-3");
-		g_hash_table_insert (eh, "iso_8859-3", "iso-8859-3");
-		g_hash_table_insert (eh, "iso8859-4", "iso-8859-4");
-		g_hash_table_insert (eh, "iso_8859-4", "iso-8859-4");
-		g_hash_table_insert (eh, "iso8859-5", "iso-8859-5");
-		g_hash_table_insert (eh, "iso_8859-5", "iso-8859-5");
-		g_hash_table_insert (eh, "iso8859-6", "iso-8859-6");
-		g_hash_table_insert (eh, "iso_8859-6", "iso-8859-6");
-		g_hash_table_insert (eh, "iso8859-7", "iso-8859-7");
-		g_hash_table_insert (eh, "iso_8859-7", "iso-8859-7");
-		g_hash_table_insert (eh, "iso8859-8", "iso-8859-8");
-		g_hash_table_insert (eh, "iso_8859-8", "iso-8859-8");
-		g_hash_table_insert (eh, "iso8859-9", "iso-8859-9");
-		g_hash_table_insert (eh, "iso_8859-9", "iso-8859-9");
-		g_hash_table_insert (eh, "iso8859-10", "iso-8859-10");
-		g_hash_table_insert (eh, "iso_8859-10", "iso-8859-10");
-		g_hash_table_insert (eh, "iso8859-13", "iso-8859-13");
-		g_hash_table_insert (eh, "iso_8859-13", "iso-8859-13");
-		g_hash_table_insert (eh, "iso8859-14", "iso-8859-14");
-		g_hash_table_insert (eh, "iso_8859-14", "iso-8859-14");
-		g_hash_table_insert (eh, "iso8859-15", "iso-8859-15");
-		g_hash_table_insert (eh, "iso_8859-15", "iso-8859-15");
-
-		/* fixme: should the following be iso-10646/ucs2? */
-		g_hash_table_insert (eh, "iso10646-1", "iso-10646-1");
-		g_hash_table_insert (eh, "iso_10646-1", "iso-10646-1");
-
-		g_hash_table_insert (eh, "koi8-r", "koi8-r");
-		g_hash_table_insert (eh, "koi8-ub", "koi8-ub");
-
-		/* fixme: Japanese and Korean users, please fix the following: */
-		g_hash_table_insert (eh, "eucjp-0", "EUC-JP");
-		g_hash_table_insert (eh, "euc-jp", "EUC-JP");
-		g_hash_table_insert (eh, "ujis-0", "ujis");
-		g_hash_table_insert (eh, "jisx0208.1983-0", "shift-jis");
-		g_hash_table_insert (eh, "jisx0212.1990-0", "shift-jis");
-		g_hash_table_insert (eh, "euckr-0", "euc-kr");
-		g_hash_table_insert (eh, "euc-kr", "euc-kr");
-
-		g_hash_table_insert (eh, "big5-0", "big5");
-		g_hash_table_insert (eh, "big5.eten-0", "big5");
-		g_hash_table_insert (eh, "big5hkscs-0", "big5hkscs");
-		g_hash_table_insert (eh, "gb2312-0", "gb2312");
-		g_hash_table_insert (eh, "gb2312.1980-0", "gb2312");
-		g_hash_table_insert (eh, "gb18030-0", "gb18030");
-		g_hash_table_insert (eh, "gbk-0", "gbk");
-	}
-
-	strncpy (e, encoding, 64);
-	e[63] = '\0';
-	g_strdown (e);
-
-	finalenc = g_hash_table_lookup (eh, e);
-	if (finalenc == NULL) {
-		finalenc = g_strdup(encoding);
-		g_hash_table_insert(eh, g_strdup(e), finalenc);
-	}
-
-	return finalenc;
-}
-
-static const gchar *
-get_locale_charset (void)
-{
-	gchar *locale;
-	static gchar *charset = NULL;
-	static gboolean initialized = FALSE;
-
-	if (!initialized) {
-		locale = gtk_set_locale ();
-		if (strcmp (locale, "C") == 0)
-			locale = NULL;
-		if (locale != NULL) {
-#ifdef HAVE_CODESET
-			charset = g_strdup(nl_langinfo(CODESET));
-			g_strdown(charset);
-#else
-			gchar * p;
-			size_t len;
-
-			p = strchr (locale, '@');
-			if (p) {
-				len = p - locale;
-			} else {
-				p = strchr (locale, ';');
-				len = p ? (p - locale) : strlen (locale);
-			}
-			p = strchr (locale, '.');
-			if (p && p < (locale + len - 1)) {
-				charset = g_strndup (p + 1, len - (p - locale) + 1);
-				g_strdown (charset);
-			}
-#endif
-		}
-		initialized = TRUE;
-	}
-
-#ifdef E_FONT_VERBOSE
-	if (e_font_verbose) {
-		g_print ("Locale charset is: %s\n", charset);
-	}
-#endif
-
-	return charset;
-}
-
-#if 0
-static const gchar *
-locale_charset_to_encoding(void)
-{
-	static GHashTable *lh = NULL;
-	const gchar *charset;
-
-	if (!lh) {
-		lh = g_hash_table_new (g_str_hash, g_str_equal);
-
-		g_hash_table_insert (lh, "eucjp", "iso8859-1");
-		g_hash_table_insert (lh, "ujis", "iso8859-1");
-	}
-	charset = get_locale_charset ();
-
-	return g_hash_table_lookup (lh, charset);
-}
-#endif
-
-const gchar *
-e_locale_encoding (void)
-{
-#ifdef HAVE_CODESET
-	const char *encoding = nl_langinfo(CODESET);
-	if (encoding) {
-		const gchar *trans_encoding = translate_encoding (encoding);
-		if (trans_encoding)
-			return trans_encoding;
-		else
-			return encoding;
-	}
-#endif
-	return NULL;
+	return e_font_utf8_text_width (font, style, text, g_utf8_skip[*(guchar *)text]);
 }
 
 const gchar *
@@ -1020,14 +856,9 @@ e_gdk_font_encoding (GdkFont *font)
 
 	if (font->type == GDK_FONT_FONTSET) {
 		/* We are fontset, so try to find right encoding from locale */
-		encoding = get_locale_charset ();
-		if (encoding) {
-			const gchar *trans_encoding = translate_encoding (encoding);
-			if (trans_encoding)
-			    return trans_encoding;
-			else
-			    return encoding;
-		}
+		encoding = e_iconv_charset_name(e_iconv_locale_charset());
+		if (encoding)
+			return encoding;
 	}
 
 	font_atom = gdk_atom_intern ("FONT", FALSE);
@@ -1069,7 +900,7 @@ e_gdk_font_encoding (GdkFont *font)
 		return NULL;
 	}
 
-	encoding = translate_encoding (p);
+	encoding = e_iconv_charset_name(p);
 	e_cache_insert (cache, font, (gpointer) encoding, 1);
 	g_free (name);
 
@@ -1077,76 +908,16 @@ e_gdk_font_encoding (GdkFont *font)
 	return encoding;
 }
 
-iconv_t
-e_iconv_from_charset (const gchar *enc)
-{
-	static GHashTable *ih = NULL;
-	iconv_t ic;
-
-	if (!enc) return (iconv_t) -1;
-
-	if (!ih) ih = g_hash_table_new (g_str_hash, g_str_equal);
-
-	ic = g_hash_table_lookup (ih, enc);
-
-	if (!ic) {
-		ic = iconv_open ("UTF-8", enc);
-		if (ic == (iconv_t) -1) return ic;
-		g_hash_table_insert (ih, (gpointer) enc, ic);
-	}
-
-	return ic;
-}
-
-iconv_t
-e_iconv_to_charset (const gchar *enc)
-{
-	static GHashTable *ih = NULL;
-	iconv_t ic;
-
-	if (!enc) return (iconv_t) -1;
-
-	if (!ih) ih = g_hash_table_new (g_str_hash, g_str_equal);
-
-	ic = g_hash_table_lookup (ih, enc);
-
-	if (!ic) {
-		ic = iconv_open (enc, "UTF-8");
-		if (ic == (iconv_t) -1) return ic;
-		g_hash_table_insert (ih, (gpointer) enc, ic);
-	}
-
-	return ic;
-}
-
-iconv_t
-e_iconv_from_locale (void)
-{
-	const gchar *enc;
-
-	enc = e_locale_encoding ();
-	return e_iconv_from_charset (enc);
-}
-
-iconv_t
-e_iconv_to_locale (void)
-{
-	const gchar *enc;
-
-	enc = e_locale_encoding ();
-	return e_iconv_to_charset (enc);
-}
-
 iconv_t
 e_iconv_from_gdk_font (GdkFont *font)
 {
 	const gchar *enc;
 
-	if (!font) return (iconv_t) -1;
-
 	enc = e_gdk_font_encoding (font);
+	if (enc == NULL)
+		return (iconv_t) -1;
 
-	return e_iconv_from_charset (enc);
+	return e_iconv_open("utf-8", enc);
 }
 
 iconv_t
@@ -1154,11 +925,11 @@ e_iconv_to_gdk_font (GdkFont *font)
 {
 	const gchar *enc;
 
-	if (!font) return (iconv_t) -1;
-
 	enc = e_gdk_font_encoding (font);
+	if (enc == NULL)
+		return (iconv_t) -1;
 
-	return e_iconv_to_charset (enc);
+	return e_iconv_open(enc, "utf-8");
 }
 
 /*
